import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;
import org.junit.Test;

/*
 * 此範例程式是採用Junit套件進行測試
 * 機密性資料如: 專案金鑰(PK4254GZMGASPHBB01)、設備編號(926957391)僅為說明用途之假資料
 * 請您遵循四大步驟，並根據您的專案資料進行修改，方可運行
 
 * 雙向認證之憑證注意事項
 *  - Server憑證是向Public CA申請而來
 *  - Client憑證是向CHT自建Private CA申請而來
 */
public class HttpsTest {

	// 1. 請填寫正確的憑證檔案位置 例如 d:/temp/926957391.pfx
	//    此Device憑證可於中華電信智慧連網大平台網站中的 "專案管理/設備管理/憑證申請" 取得
	File pfx = new File("d:/temp/ECC_926957391.pfx");
	
	// 此pfx有密碼保護，預設為您的設備編號(Ex:926957391)
	char[] key = "".toCharArray();
	
	KeyStore newKeyStoreByPfx() throws GeneralSecurityException, IOException {		
		KeyStore ks = KeyStore.getInstance("PKCS12");	
		FileInputStream fis = new FileInputStream(pfx);
		try {		
			ks.load(fis, key);
			return ks;			
			
		} finally {
			fis.close();
		}
	}
	
	/* 4.設定完前三步驟後，即可進行單元測試，驗證連線成功與否
		本範例若填寫正確之資料，則會獲得類似如下之Json結果 [{"id":"Sensor01","name":"Sensor01","desc":"some desc","type":"gauge","uri":"FB919AAUXWHY4SEBY","unit":"","formula":"","attributes":[]}]
	*/
	@Test
	public void test() throws Exception {
		
		X509TrustManager combine_TM = null;
		
		/*  
		 * Server Certificate
		 * Get the java library default trusted
		 * Keep these default trusted manager because CHT-IOT server's certificate is released by a valid and public CA Server
		 */
		
		TrustManagerFactory server_TMF = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
		server_TMF.init((KeyStore) null);
		
		X509TrustManager server_TM = null;
		for (TrustManager tm : server_TMF.getTrustManagers()) {
		    if (tm instanceof X509TrustManager) {
		    		server_TM = (X509TrustManager) tm;
		    		combine_TM = (X509TrustManager) tm;
		        break;
		    }
		}
				
		/* 
		 * Client Certificate
		 * Get the device certificate
		 * Keep these privately trusted manager because CHT-IOT client's certificate is released by a private CA Server
		 */
		
		KeyStore ks = newKeyStoreByPfx();
		KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
		kmf.init(ks, key);
		
		TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
		tmf.init(ks);		
				
		X509TrustManager client_TM = null;
		for (TrustManager tm : tmf.getTrustManagers()) {
		    if (tm instanceof X509TrustManager) {
		        client_TM = (X509TrustManager) tm;
		        combine_TM = (X509TrustManager) tm;
		        break;
		    }
		}
				
		final X509TrustManager finalServerTM = server_TM;
		final X509TrustManager finalClientTM = client_TM;
		final X509TrustManager finalCombineTM = combine_TM;
		
		X509TrustManager custom_TM = new X509TrustManager() {
		    @Override
		    public X509Certificate[] getAcceptedIssuers() {
		        return finalCombineTM.getAcceptedIssuers();
		    }

		    @Override
		    public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
		        try {
		            finalClientTM.checkServerTrusted(chain, authType);
		        } catch (CertificateException e) {
		            finalServerTM.checkServerTrusted(chain, authType);
		        }
		    }

		    @Override
		    public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
		        finalServerTM.checkClientTrusted(chain, authType);
		        finalClientTM.checkClientTrusted(chain, authType);
		    }
		};
		
		SSLContext ssl = SSLContext.getInstance("TLS");
		ssl.init(kmf.getKeyManagers(), new TrustManager[] {custom_TM}, new SecureRandom());
		
		HttpsURLConnection.setDefaultSSLSocketFactory(ssl.getSocketFactory());		
		
		/* 2. 請參考API文件 http://iot.cht.com.tw/iot/developer/api 選擇適合您的API來進行雙向認證測試
		   本範例為獲取感測資料之API，故使用GET方法
		   
		   協定必為 https
		   埠號必為 7443
		*/
		URL url = new URL("https://iot.cht.com.tw:7443/iot/v1/device/926957391/sensor");	
		HttpURLConnection con = (HttpURLConnection) url.openConnection();		
		con.setRequestMethod("GET");
		
		// 3. 設置正確的專案金鑰，例如: PK9254GZMGBSPHAB01
		//    專案金鑰可從 "專案管理/編輯專案/權限資料/內容" 中找到
		//    CK 為保留字不可更動
		con.setRequestProperty("CK", "PK9254GZMGBSPHAB01");
		
		con.connect();
		
		int sc = con.getResponseCode();
		if (sc != 200) {
			throw new IOException(String.format("[%d] %s", sc, con.getResponseMessage()));
		}
		else{
			System.out.println("Successful");
			InputStreamReader isr = new InputStreamReader(con.getInputStream(), "UTF-8");
			BufferedReader br = new BufferedReader(isr);
			String ln;
			while ((ln = br.readLine()) != null) {
				System.out.println(ln);
			}
		}		
				
	}

}
